import { CamelCase, PascalCase } from 'type-fest';
import * as _ from 'lodash';

export type CamelCasedProps<T> = {
  [K in keyof T as CamelCase<K>]: CamelCasedProps<T[K]>;
};

export function pascalCase<T extends string>(str: T): PascalCase<T> {
  return _.startCase(_.camelCase(str)).replace(/ /g, '') as PascalCase<T>;
}

export function pascalCaseProperty<T extends Record<string, any>>(
  obj: T,
): { [key in PascalCase<Extract<keyof T, string>>]: any } {
  let res: Record<string, any> = {};
  if (_.isObject(obj)) {
    res = {} as T;
    Object.keys(obj).forEach((key: string) => {
      const val = obj[key];
      const k = pascalCase(key);
      res[k] = _.isObject(val) || _.isArray(val) ? pascalCaseProperty(val) : val;
    });
  }
  if (_.isArray(obj as any)) {
    res = [];
    obj.forEach((item: any) => {
      res.push(_.isObject(item) || _.isArray(item) ? pascalCaseProperty(item) : item);
    });
  }
  return res as T;
}

export function removeUndefined<T>(obj: T): T {
  return JSON.parse(JSON.stringify(obj));
}

export function deepClone<T>(obj: T): T {
  return JSON.parse(JSON.stringify(obj));
}
